close all 
clear all
clc

% opening solvent database
filename = uigetfile('*.txt');

% to open an exact file
%filename = 'template.txt';

delimiter = '\t';
tdfread(filename,delimiter);
txt_rem = '.txt';
filename1 = regexprep(filename, txt_rem, '');
   
% define solvents and corresponding dispersion component, polar component, and hydrogen bonding component

solvent.name = Solvent;
solvent.T = T;
solvent.d = 2.*dd; % hansen plot uses 2*dispersion component
solvent.p = dp;
solvent.h = dh;
solvent.using = Used;
solvent.worked = Worked;

% plot formating 
cmap = hsv(length(solvent.using));
col=[];

%% plots of the used solvents. Here all the solvents in the list will be shown in the hansen plot
solvent_used = 0;

figure
subplot(1,2,1)
for i=1:length(solvent.using)
    if (strcmp(solvent.using(i),'y'))
        % creating struct of used solvents 
        solvent_used = solvent_used + 1;
        
        used.name(solvent_used,:) = Solvent(i,:);
        used.T(solvent_used) = T(i);
        used.d(solvent_used) = 2.*dd(i);
        used.p(solvent_used) = dp(i);
        used.h(solvent_used) = dh(i);
        used.using(solvent_used) = Used(i);
        used.worked(solvent_used) = Worked(i);   
        col(solvent_used,:) = cmap(i,:);
        
        plot3(used.d(solvent_used),used.p(solvent_used),used.h(solvent_used),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,'MarkerFaceColor',col(solvent_used,:),...
                'MarkerSize',8);
        text(used.d(solvent_used)+1,used.p(solvent_used),used.h(solvent_used),num2str(solvent_used))
        
        hold on
          

    end
end

hold off
grid on
axis square
set(gca,'FontSize',18,'Linewidth',1)
daspect([1,1,1]);

% add title and axis labels
title('Solvents','FontSize',18);
xlabel('2D (MPa^{0.5})','FontSize',18);
ylabel('P (MPa^{0.5})','FontSize',18);
zlabel('H (MPa^{0.5})','FontSize',18);

subplot(1,2,2)
for i=1:solvent_used
     plot(0,solvent_used-i,'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,'MarkerFaceColor',col(i,:),...
                'MarkerSize',8);
     text(0-.2,solvent_used-i,num2str(i));
     text(0+.1,solvent_used-i,used.name(i,:));
     hold on
end

axis off
filename_solvents = strcat(filename1,'_solvents');
%print ('-dtiff', filename_solvents, '-r300', '-cmyk');
print ('-dpdf', filename_solvents, '-r300', '-cmyk');

%% separating solvents that worked and didn't work

solvent_worked = 0;
solvent_didnt_work = 0;

for i=1:solvent_used
    if (strcmp(used.worked(i),'y'))
        
        solvent_worked = solvent_worked +1;
        
        sworked.name(solvent_worked,:) = used.name(i,:);
        sworked.T(solvent_worked) = used.T(i);
        sworked.d(solvent_worked) = used.d(i);
        sworked.p(solvent_worked) = used.p(i);
        sworked.h(solvent_worked) = used.h(i);
        sworked.using(solvent_worked) = used.using(i);
        sworked.worked(solvent_worked) = used.worked(i);
        
    else
            
        solvent_didnt_work = solvent_didnt_work +1;
        
        sdidntwork.name(solvent_didnt_work,:) = used.name(i,:);
        sdidntwork.T(solvent_didnt_work) = used.T(i);
        sdidntwork.d(solvent_didnt_work) = used.d(i);
        sdidntwork.p(solvent_didnt_work) = used.p(i);
        sdidntwork.h(solvent_didnt_work) = used.h(i);
        sdidntwork.using(solvent_didnt_work) = used.using(i);
        sdidntwork.worked(solvent_didnt_work) = used.worked(i);

    end
end


% creating hansen sphere by fitting the solvents that worked

X=[sworked.d' sworked.p' sworked.h'];
[Center_LSE,Radius_LSE] = minboundsphere(X);

h.center = Center_LSE;
h.radius = Radius_LSE;
 
figure
subplot(1,2,1)
for i=1:solvent_worked
    lw(i)= plot3(sworked.d(i),sworked.p(i),sworked.h(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...
                'MarkerFaceColor',[77, 175, 74]./255,...
                'MarkerSize',8);
    %text(sworked.d(i),sworked.p(i),sworked.h(i),sworked.name(i,:))   
        
    hold on
end

hold on 

for i=1:solvent_didnt_work
    ldw(i)= plot3(sdidntwork.d(i),sdidntwork.p(i),sdidntwork.h(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...%'MarkerFaceColor',Red,...%'r',...%[.49 1 .63],...
                'MarkerSize',8);
    %text(sdidntwork.d(i),sdidntwork.p(i),sdidntwork.h(i),sdidntwork.name(i,:))
            
    hold on
end

hold on 

[Base_X,Base_Y,Base_Z] = sphere(256);

surf(Radius_LSE*Base_X+Center_LSE(1),...
    Radius_LSE*Base_Y+Center_LSE(2),...
    Radius_LSE*Base_Z+Center_LSE(3),'FaceAlpha',.5,'FaceColor',[234, 234, 234]./255,'EdgeColor','k','EdgeAlpha',.05)

  
%view([45,28])

% add title and axis labels
title('Hansen Plot','FontSize',18);
xlabel('2D (MPa^{0.5})','FontSize',18);
ylabel('P (MPa^{0.5})','FontSize',18);
zlabel('H (MPa^{0.5})','FontSize',18);

hold off
grid on
axis square
set(gca,'FontSize',18,'Linewidth',1)
daspect([1,1,1]);

filename_hansen = strcat(filename1,'_hansen');
%print ('-dtiff', filename_hansen, '-r300', '-cmyk');
print ('-dpdf', filename_hansen, '-r300', '-cmyk');

%% 2d plots xy

figure;clf;
subplot(2,2,1)
for i=1:solvent_worked
    plot(sworked.d(i),sworked.p(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...
                'MarkerFaceColor',[77, 175, 74]./255,...
                'MarkerSize',8);
    %text(sworked.d(i),sworked.p(i),sworked.h(i),sworked.name(i,:))   
        
    hold on
end

hold on 

for i=1:solvent_didnt_work
    plot(sdidntwork.d(i),sdidntwork.p(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...%'MarkerFaceColor',Red,...%'r',...%[.49 1 .63],...
                'MarkerSize',8);
    %text(sdidntwork.d(i),sdidntwork.p(i),sdidntwork.h(i),sdidntwork.name(i,:))
            
    hold on
end

hold on 
circle(h.center(1),h.center(2),h.radius)%x,y,r
axis equal

set(gca,'FontSize',18,'Linewidth',1)
xlabel('2D (MPa^{0.5})','FontSize',18);
ylabel('P (MPa^{0.5})','FontSize',18);


%

subplot(2,2,2)
for i=1:solvent_worked
    plot(sworked.p(i),sworked.h(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...
                'MarkerFaceColor',[77, 175, 74]./255,...
                'MarkerSize',8);
    %text(sworked.d(i),sworked.p(i),sworked.h(i),sworked.name(i,:))   
        
    hold on
end

hold on 

for i=1:solvent_didnt_work
    plot(sdidntwork.p(i),sdidntwork.h(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...%'MarkerFaceColor',Red,...%'r',...%[.49 1 .63],...
                'MarkerSize',8);
    %text(sdidntwork.d(i),sdidntwork.p(i),sdidntwork.h(i),sdidntwork.name(i,:))
            
    hold on
end

hold on 
circle(h.center(2),h.center(3),h.radius)%x,y,r
axis equal

set(gca,'FontSize',18,'Linewidth',1)
xlabel('P (MPa^{0.5})','FontSize',18);
ylabel('H (MPa^{0.5})','FontSize',18);

%

subplot(2,2,3)
for i=1:solvent_worked
    plot(sworked.h(i),sworked.d(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...
                'MarkerFaceColor',[77, 175, 74]./255,...
                'MarkerSize',8);
    %text(sworked.d(i),sworked.p(i),sworked.h(i),sworked.name(i,:))   
        
    hold on
end

hold on 

for i=1:solvent_didnt_work
    plot(sdidntwork.h(i),sdidntwork.d(i),'ko',...%'MarkerEdgeColor','k',...
                'Linewidth',1,...%'MarkerFaceColor',Red,...%'r',...%[.49 1 .63],...
                'MarkerSize',8);
    %text(sdidntwork.d(i),sdidntwork.p(i),sdidntwork.h(i),sdidntwork.name(i,:))
            
    hold on
end

hold on 
circle(h.center(3),h.center(1),h.radius)%x,y,r
axis equal

set(gca,'FontSize',18,'Linewidth',1)
xlabel('H (MPa^{0.5})','FontSize',18);
ylabel('2D (MPa^{0.5})','FontSize',18);

subplot(2,2,4)

text(0,2*.4,strcat('\delta_D = ',num2str(h.center(1)/2)),'FontSize',18);
text(0,2*.3,strcat('\delta_P = ',num2str(h.center(2))),'FontSize',18);
text(0,2*.2,strcat('\delta_H = ',num2str(h.center(3))),'FontSize',18);
text(0,2*.1,strcat('R = ',num2str(h.radius)),'FontSize',18);

axis off

filename_2d = strcat(filename1,'_2d');
%print ('-dtiff', filename_2d, '-r300', '-cmyk');
print ('-dpdf', filename_2d, '-r300', '-cmyk');


%% hansen distances

for i=1:solvent_worked
    sworked.ra(i) = sqrt((sworked.d(i) - h.center(1))^2 + (sworked.p(i) - h.center(2))^2 + (sworked.h(i) - h.center(3))^2);
    sworked.red(i) = sworked.ra(i)/h.radius;    
    sworked.red(i) = ceil(sworked.red(i)*100)/100;
end

for i=1:solvent_didnt_work
    sdidntwork.ra(i) = sqrt((sdidntwork.d(i) - h.center(1))^2 + (sdidntwork.p(i) - h.center(2))^2 + (sdidntwork.h(i) - h.center(3))^2);
    sdidntwork.red(i) = sdidntwork.ra(i)/h.radius; 
    sdidntwork.red(i) = ceil(sdidntwork.red(i)*100)/100;
end

%% printing report 

filename_solvents_report = strcat(filename1,'_report.txt');

fname = vertcat(sworked.name, sdidntwork.name);
fT = [sworked.T, sdidntwork.T];
fd = [sworked.d, sdidntwork.d]; 
fp = [sworked.p, sdidntwork.p]; 
fh = [sworked.h, sdidntwork.h]; 
fworked = [sworked.worked, sdidntwork.worked]; 
fred = [sworked.red, sdidntwork.red]; 

fdata = struct('Solvent',fname,'T',fT','dd',fd','dp',fp','dh',fh','Solubility',fworked','RED',fred');
tdfwrite(filename_solvents_report,fdata);

%%

h = msgbox('Hansen calculations are completed, please check the output files.');
disp('Hansen calculations are completed, please check the output files.');

